import numpy as np
from casadi import SX, Function

from math import isnan, sqrt, ceil, inf
import matplotlib

import matplotlib.pyplot as plt


def plot_trajectories(s_trajs, a_trajs, labels):

    if not isinstance(s_trajs, list):
        s_trajs = [s_trajs]
        a_trajs = [a_trajs]
        labels = [labels]
    
    prop_cycle = plt.rcParams['axes.prop_cycle']
    colors = prop_cycle.by_key()['color']

    N = a_trajs[0].size
    ts = np.arange(0, N+1)

    # uncomment for nice latex style plots
    # may not work on every computer
    # latexify()

    plt.figure(figsize=(5, 8))
    plt.subplot(3, 1, 1)
    for i, x_traj in enumerate(s_trajs):
        plt.plot(ts, x_traj[0, :].T, '-', alpha=0.7, color=colors[i], label=labels[i])
    plt.ylabel(r'$\theta_k$')
    plt.grid()
    plt.legend()

    plt.subplot(3, 1, 2)
    for i, x_traj in enumerate(s_trajs):
        plt.plot(ts, x_traj[1, :].T, '-', alpha=0.7, color=colors[i], label=labels[i])
    plt.ylabel(r'$\omega_k$')
    plt.grid()
    plt.legend()

    plt.subplot(3, 1, 3)
    for i, u_traj in enumerate(a_trajs):
        plt.step(ts[:-1], u_traj.T, alpha=0.7, color=colors[i], label=labels[i], where='post')
    plt.grid()
    plt.xlabel(r'time step $k$')
    plt.ylabel(r'$a_k$')
    plt.legend()


def latexify(fig_width=None, fig_height=None):
    """Set up matplotlib's RC params for LaTeX plotting.
    Call this before plotting a figure.

    Parameters
    ----------
    fig_width : float, optional, inches
    fig_height : float,  optional, inches
    columns : {1, 2}
    """

    # code adapted from http://www.scipy.org/Cookbook/Matplotlib/LaTeX_Examples

    # Width and max height in inches for IEEE journals taken from
    # computer.org/cms/Computer.org/Journal%20templates/transactions_art_guide.pdf

    if fig_width is None:
        fig_width = 5  # width in inches

    if fig_height is None:
        golden_mean = (sqrt(5)-1.0)/2.0    # Aesthetic ratio
        fig_height = fig_width*golden_mean # height in inches


    params = {'backend': 'ps',
              'text.latex.preamble': r"\usepackage{gensymb} \usepackage{amsmath}",
              'axes.labelsize': 10, # fontsize for x and y labels (was 10)
              'axes.titlesize': 10,
              'legend.fontsize': 10, # was 10
              'xtick.labelsize': 10,
              'ytick.labelsize': 10,
              'text.usetex': True,
              'figure.figsize': [fig_width, fig_height],
              'font.family': 'serif'
    }

    matplotlib.rcParams.update(params)
